/*****************************************************************************

*
* File:    le_central.c
* Summary: LE Central
*
*/

#ifdef __cplusplus
extern "C"
{
#endif

/* Standard Includes */
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

/* Includes */
#include "le_central.h"

#include "app_error.h"
#include "app_timer.h"
#include "app_util.h"
#include "ble.h"
#include "ble_db_discovery.h"
#include "ble_gap.h"
#include "ble_hci.h"
#include "ble_nus_c.h"
#include "le_central.h"
#include "main.h"
#include "nordic_common.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_scan.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_sdh.h"
#include "nrf_sdh_ble.h"
#include "nrf_sdh_soc.h"
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

#include "ble_aqss.h"
#include "ble_aqss_c.h"
#include "ble_aqts.h"
#include "ble_aqts_c.h"
#include "ble_dis_c.h"
#include "task_repeater.h"
#include "repeater.h"

#define SCAN_DURATION_WITELIST    3000                             /**< Duration of the scanning in units of 10 milliseconds. */

BLE_DB_DISCOVERY_DEF(m_db_disc);                                    /**< DB discovery module instance. */

NRF_BLE_SCAN_DEF(m_scan);                                               /**< Scanning Module instance. */
BLE_AQSS_C_ARRAY_DEF(
        m_ble_aqss_c, NRF_SDH_BLE_CENTRAL_LINK_COUNT
); /**< Aquai Shower Service client instance. */
BLE_AQTS_C_ARRAY_DEF(
        m_ble_aqts_c, NRF_SDH_BLE_CENTRAL_LINK_COUNT
); /**< Aquai Toilet Service client instance. */
BLE_DIS_C_ARRAY_DEF(m_ble_dis_c, NRF_SDH_BLE_CENTRAL_LINK_COUNT
);


NRF_BLE_GQ_DEF(m_ble_gatt_queue,                                        /**< BLE GATT Queue instance. */
NRF_SDH_BLE_CENTRAL_LINK_COUNT, NRF_BLE_GQ_QUEUE_SIZE
);

extern char *array_device_list[5][40];

/**@brief AQSS UUID. */
static ble_uuid_t const m_aqss_uuid = {
        .uuid = BLE_UUID_AQSS_SERVICE, .type = AQSS_SERVICE_UUID_TYPE};

/**@brief AQTS UUID. */
static ble_uuid_t const m_aqts_uuid = {
        .uuid = BLE_UUID_AQTS_SERVICE, .type = AQTS_SERVICE_UUID_TYPE};

static ble_uuid_t const m_dis_uuid = {
        .uuid = BLE_UUID_DEVICE_INFORMATION_SERVICE, .type = BLE_UUID_TYPE_BLE};

static ble_gap_scan_params_t
        m_scan_param = /**< Scan parameters requested for scanning and connection. */
        {
    .active        = 0x01,
    .interval      = NRF_BLE_SCAN_SCAN_INTERVAL,
    .window        = NRF_BLE_SCAN_SCAN_WINDOW,
    .filter_policy = BLE_GAP_SCAN_FP_WHITELIST,
    .timeout       = SCAN_DURATION_WITELIST,
    .scan_phys     = BLE_GAP_PHY_1MBPS,
        };

test_device_t test_toilet_device_list[NRF_SDH_BLE_CENTRAL_LINK_COUNT];
test_device_t test_shower_device_list[NRF_SDH_BLE_CENTRAL_LINK_COUNT];
test_device_t m_aq_device[NRF_SDH_BLE_CENTRAL_LINK_COUNT];

ble_aqss_c_t *get_shower_le_dev_instance(uint16_t conn_handle) {
    return &m_ble_aqss_c[conn_handle];
}

ble_aqts_c_t *get_toilet_le_dev_instance(uint16_t conn_handle) {
    return &m_ble_aqts_c[conn_handle];
}



test_device_t *get_aq_dev_instance(uint16_t conn_handle) { return &m_aq_device[conn_handle]; }

/**@brief Function for handling database discovery events.
 *
 * @details This function is a callback function to handle events from the database discovery
 * module. Depending on the UUIDs that are discovered, this function forwards the events to
 * their respective services.
 *
 * @param[in] p_event  Pointer to the database discovery event.
 */
static void db_disc_handler(ble_db_discovery_evt_t *p_evt) {
    NRF_LOG_DEBUG("Disc Handler: evt type: %d", p_evt->evt_type);
    ble_dis_c_on_db_disc_evt(&m_ble_dis_c[p_evt->conn_handle], p_evt);
    ble_aqss_c_on_db_disc_evt(&m_ble_aqss_c[p_evt->conn_handle], p_evt);
    ble_aqts_c_on_db_disc_evt(&m_ble_aqts_c[p_evt->conn_handle], p_evt);
}


/**@brief Function for handling BLE Stack events that are related to central application.
 *
 * @details This function keeps the connection handles of central application up-to-date. It
 * parses scanning reports, initiates a connection attempt to peripherals when a target UUID
 * is found, and manages connection parameter update requests. Additionally, it updates the
 * status of LEDs used to report the central application's activity.
 *
 * @note        Since this function updates connection handles, @ref BLE_GAP_EVT_DISCONNECTED
 * events must be dispatched to the target application before invoking this function.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
void on_ble_central_evt(ble_evt_t const *p_ble_evt) {
    ble_gap_evt_t const *p_gap_evt = &p_ble_evt->evt.gap_evt;
    ret_code_t            err_code;


    switch (p_ble_evt->header.evt_id) {
        // Upon connection, check which peripheral is connected (HR or RSC), initiate DB
        //  discovery, update LEDs status, and resume scanning, if necessary.
        case BLE_GAP_EVT_CONNECTED: {
            NRF_LOG_INFO("CENTRAL: Connected, handle: %d.", p_gap_evt->conn_handle);
		//memcpy(send_device_list[0], &p_ble_evt->evt.gap_evt.params.adv_report.data.p_data[2], 22);
            // Discover peer's services.
            err_code = ble_db_discovery_start(&m_db_disc, p_ble_evt->evt.gap_evt.conn_handle);
            if(err_code != NRF_ERROR_BUSY) {
                APP_ERROR_CHECK(err_code);
            }

            if (ble_conn_state_central_conn_count() < NRF_SDH_BLE_CENTRAL_LINK_COUNT) {
                scan_start();
            }
        }
            break; // BLE_GAP_EVT_CONNECTED

        // Upon disconnection, reset the connection handle of the peer that disconnected, update
        // the status of LEDs, and start scanning again.
        case BLE_GAP_EVT_DISCONNECTED: {
            NRF_LOG_INFO("CENTRAL: Disconnected, handle: %d, reason: 0x%x",
                         p_gap_evt->conn_handle, p_gap_evt->params.disconnected.reason);
            //connect_count--;
            /* Send  RSSI = -100 to indicate disconnected for conn_handle */
            repeater_rssi_update(p_gap_evt->conn_handle, -100);

            /* Clear test device context */
            NRF_LOG_INFO("CENTRAL: Clear test device context");
            m_aq_device[p_gap_evt->conn_handle].device_model = NO_MODEL;
            m_aq_device[p_gap_evt->conn_handle].conn_handle = BLE_CONN_HANDLE_INVALID;
            
            memset(m_aq_device[p_gap_evt->conn_handle].device_id, 0x00, sizeof(m_aq_device[p_gap_evt->conn_handle].device_id));

            memset(m_aq_device[p_gap_evt->conn_handle].device_model_name, 0x00, 7);

            if (ble_conn_state_central_conn_count() < NRF_SDH_BLE_CENTRAL_LINK_COUNT) {
                scan_start();
            }
        }
            break; // BLE_GAP_EVT_DISCONNECTED

        case BLE_GAP_EVT_TIMEOUT: {
            // Timeout for scanning is not specified, so only connection attemps can time out.
            if (p_gap_evt->params.timeout.src == BLE_GAP_TIMEOUT_SRC_CONN) {
                NRF_LOG_DEBUG("CENTRAL: Connection Request timed out.");
            }
        }
            break;

        case BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST: {
            // Accept parameters requested by peer.
            err_code = sd_ble_gap_conn_param_update(p_gap_evt->conn_handle,
                                        &p_gap_evt->params.conn_param_update_request.conn_params);
            APP_ERROR_CHECK(err_code);
        }
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            NRF_LOG_DEBUG("CENTRAL: GATT Client Timeout.");
            NRF_LOG_INFO("BLE_GATTC_EVT_TIMEOUT");
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            NRF_LOG_DEBUG("CENTRAL: GATT Server Timeout.");
            NRF_LOG_INFO("BLE_GATTS_EVT_TIMEOUT");
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_GAP_EVT_RSSI_CHANGED: {
            int8_t rssi;
            uint8_t chn_index;
            sd_ble_gap_rssi_get(p_gap_evt->conn_handle, &rssi, &chn_index);
            repeater_rssi_update(p_gap_evt->conn_handle, rssi);
            break;
        }
       case BLE_GAP_EVT_ADV_REPORT:
	{
          char name[32];
	  uint16_t offset = 0;
	  uint16_t length = ble_advdata_search(p_ble_evt->evt.gap_evt.params.adv_report.data.p_data, p_ble_evt->evt.gap_evt.params.adv_report.data.len, &offset, BLE_GAP_AD_TYPE_COMPLETE_LOCAL_NAME);
	 // if(length == 0){
		//length = ble_advdata_search(p_ble_evt->evt.gap_evt.params.adv_report.data.p_data, p_ble_evt->evt.gap_evt.params.adv_report.data.len, &offset, BLE_GAP_AD_TYPE_SHORT_LOCAL_NAME);
		//}

       	if(length != 0){
		memcpy(name, &p_ble_evt->evt.gap_evt.params.adv_report.data.p_data[offset], length);
               // name[length] = '\0';
        	//scan_debug NRF_LOG_INFO("333 name: %s - %d, offset - %d\n", name,length,offset);
		}
         break;
	}
        default:
            // No implementation needed.
            break;
    }
}

/**@brief Function for handling Scanning Module events.
 */
static void scan_evt_handler(scan_evt_t const *p_scan_evt) {
    ret_code_t err_code;

    switch (p_scan_evt->scan_evt_id) {
    case NRF_BLE_SCAN_EVT_CONNECTING_ERROR: {
        err_code = p_scan_evt->params.connecting_err.err_code;
        APP_ERROR_CHECK(err_code);
    }
    break;

    case NRF_BLE_SCAN_EVT_CONNECTED: {
            ble_gap_evt_connected_t const *p_connected =
            p_scan_evt->params.connected.p_connected;
        // Scan is automatically stopped by the connection.
            NRF_LOG_INFO("Connecting to target %02x%02x%02x%02x%02x%02x",
                         p_connected->peer_addr.addr[0], p_connected->peer_addr.addr[1],
                         p_connected->peer_addr.addr[2], p_connected->peer_addr.addr[3],
                         p_connected->peer_addr.addr[4], p_connected->peer_addr.addr[5]);
    }
    break;

    case NRF_BLE_SCAN_EVT_SCAN_TIMEOUT: {
        NRF_LOG_INFO("Scan timed out.");
        scan_start();
    }
    break;

    default:
        break;
    }
}


/**@brief Function to start scanning. */
void scan_start(void) {
    ret_code_t ret;

    ret = nrf_ble_scan_start(&m_scan);
    APP_ERROR_CHECK(ret);
}



void scan_init_new(void) {
    ret_code_t ret;
    nrf_ble_scan_init_t init_scan;
    ret_code_t          err_code;

        for (int i = 0; i < 5; ++i) 
        {
            NRF_LOG_INFO("list = %s ", array_device_list[i]);
        }


  //if(received_list == 1)
  //{
    err_code = nrf_ble_scan_filters_disable(&m_scan);
    APP_ERROR_CHECK(err_code);


    memset(&init_scan, 0, sizeof(init_scan));

    init_scan.connect_if_match = true;
    init_scan.conn_cfg_tag     = APP_BLE_CONN_CFG_TAG;

    err_code = nrf_ble_scan_init(&m_scan, &init_scan, scan_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_UUID_FILTER, &m_aqss_uuid);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_UUID_FILTER, &m_aqts_uuid);
    APP_ERROR_CHECK(err_code);



   for (int i=0; i< NRF_BLE_SCAN_NAME_CNT; i++){
     err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_NAME_FILTER, array_device_list[i]);
    APP_ERROR_CHECK(err_code);
    }

    //err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_NAME_FILTER, arr1);
    //APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filters_enable(&m_scan, NRF_BLE_SCAN_NAME_FILTER && NRF_BLE_SCAN_UUID_FILTER, false);
    APP_ERROR_CHECK(err_code);

   
  //}
}

/**@brief Function to stop scanning. */
void scan_stop(void) {
  nrf_ble_scan_stop();
}
#define NUMBER_OF_STRING 5
#define MAX_STRING_SIZE 40



/**@brief Function for initializing the scanning and setting the filters.
only for testing purpose
 */
void scan_init(void) {
    ret_code_t          err_code;
    nrf_ble_scan_init_t init_scan;

    char arr[NUMBER_OF_STRING][MAX_STRING_SIZE] =
    { "test-AQ100WCD00000011",
      "test-AQ100WCD00000012",
      "test-AQ100WCD00000013",
      "test-AQ100WCD00000014"
    };

    memset(&init_scan, 0, sizeof(init_scan));

    init_scan.connect_if_match = true;
    init_scan.conn_cfg_tag     = APP_BLE_CONN_CFG_TAG;

    err_code = nrf_ble_scan_init(&m_scan, &init_scan, scan_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_UUID_FILTER, &m_aqss_uuid);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_UUID_FILTER, &m_aqts_uuid);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_scan_filter_set(&m_scan, SCAN_NAME_FILTER, arr);
    APP_ERROR_CHECK(err_code);

   // err_code = nrf_ble_scan_filters_enable(&m_scan, NRF_BLE_SCAN_UUID_FILTER, false);
    err_code = nrf_ble_scan_filters_enable(&m_scan, NRF_BLE_SCAN_NAME_FILTER && NRF_BLE_SCAN_UUID_FILTER, false);
   // err_code = nrf_ble_scan_filters_enable(&m_scan, NRF_BLE_SCAN_NAME_FILTER, false);

    //err_code = nrf_ble_scan_filters_enable(&m_scan, NRF_BLE_SCAN_ALL_FILTER, true);
    APP_ERROR_CHECK(err_code);

    err_code = sd_ble_gap_tx_power_set(BLE_GAP_TX_POWER_ROLE_SCAN_INIT, BLE_CONN_HANDLE_INVALID, 8); 
    APP_ERROR_CHECK(err_code);

}

/** @brief Function for initializing the database discovery module. */
void db_discovery_init(void) {
    ble_db_discovery_init_t db_init;

    memset(&db_init, 0, sizeof(ble_db_discovery_init_t));

    db_init.evt_handler  = db_disc_handler;
    db_init.p_gatt_queue = &m_ble_gatt_queue;

    ret_code_t err_code = ble_db_discovery_init(&db_init);
    APP_ERROR_CHECK(err_code);
}

repeater_task_evt_t rtevt;

/**@brief Function for handling notification received by the Aquai Shower Service (AQSS).
 *
 * @details This function takes a list of characters of length data_len and prints the
 * characters out on UART. If @ref ECHOBACK_BLE_UART_DATA is set, the data is sent back to
 * sender.
 */
static void ble_aqss_chars_received_data(
        uint16_t conn_handle, uint8_t *p_data, uint16_t data_len) {
    
    rtevt.evt_id = REPEATER_TASK_EVT_SHOWER_DATA;
    rtevt.conn_handle = conn_handle;
    if (data_len < MAX_FRAME_PAYLOAD_SIZE) {
        memcpy(rtevt.payload, p_data, data_len);
        rtevt.len = data_len;
    } else {
        memcpy(rtevt.payload, p_data, MAX_FRAME_PAYLOAD_SIZE);
        rtevt.len = MAX_FRAME_PAYLOAD_SIZE;
        NRF_LOG_INFO("%d Error: Payload too large: %d", __LINE__, data_len);
    }
    repeater_task_queue_event(&rtevt);
}

/**@brief Function for handling notification received by the Aquai Toilet Service (AQTS).
 *
 * @details This function takes a list of characters of length data_len and prints the
 * characters out on UART. If @ref ECHOBACK_BLE_UART_DATA is set, the data is sent back to
 * sender.
 */
static void ble_aqts_chars_received_data(
        uint16_t conn_handle, uint8_t *p_data, uint16_t data_len) {
    
    rtevt.evt_id = REPEATER_TASK_EVT_TOILET_DATA;
    rtevt.conn_handle = conn_handle;
    if (data_len < MAX_FRAME_PAYLOAD_SIZE) {
        memcpy(rtevt.payload, p_data, data_len);
        rtevt.len = data_len;
    } else {
        memcpy(rtevt.payload, p_data, MAX_FRAME_PAYLOAD_SIZE);
        NRF_LOG_INFO("%d Error: Payload too large: %d", __LINE__, data_len);
        rtevt.len = MAX_FRAME_PAYLOAD_SIZE;
    }
    repeater_task_queue_event(&rtevt);    
}

/**@brief Callback handling Aquai Toilet Service (AQTS) client events.
 *
 * @details This function is called to notify the application of AQTS client events.
 *
 * @param[in]   p_ble_aqts_c   AQTS client handle. This identifies the AQTS client.
 * @param[in]   p_ble_aqts_evt Pointer to the AQTS client event.
 */

/**@snippet [Handling events from the ble_nus_c module] */
static void ble_aqts_c_evt_handler(
        ble_aqts_c_t *p_ble_aqts_c, ble_aqts_c_evt_t const *p_ble_aqts_evt) {
    ret_code_t err_code;

    switch (p_ble_aqts_evt->evt_type) {
    case BLE_AQTS_C_EVT_DISCOVERY_COMPLETE:
        NRF_LOG_INFO("Discovery complete.");
            err_code = ble_aqts_c_handles_assign(
                    p_ble_aqts_c, p_ble_aqts_evt->conn_handle, &p_ble_aqts_evt->handles);
        APP_ERROR_CHECK(err_code);

            if (m_aq_device[p_ble_aqts_evt->conn_handle].device_id[0] != 0x00) {
        err_code = ble_aqts_c_tx_notif_enable(p_ble_aqts_c);
        APP_ERROR_CHECK(err_code);
                NRF_LOG_INFO("Connected to AQ1xxWC device with AQTS Service.");
            }

            m_aq_device[p_ble_aqts_evt->conn_handle].conn_handle = p_ble_aqts_evt->conn_handle;
            m_aq_device[p_ble_aqts_evt->conn_handle].device_model = AQ_1xx_WC;

        break;

    case BLE_AQTS_C_EVT_AQTS_TX_EVT:
            ble_aqts_chars_received_data(
                    p_ble_aqts_c->conn_handle, p_ble_aqts_evt->p_data, p_ble_aqts_evt->data_len);
        break;

    case BLE_AQTS_C_EVT_DISCONNECTED:
        NRF_LOG_INFO("Disconnected AQTS.");
        scan_start();
        break;
    }
}

/**@brief Callback handling Aquai Shower Service (AQSS) client events.
 *
 * @details This function is called to notify the application of AQSS client events.
 *
 * @param[in]   p_ble_aqss_c   AQSS client handle. This identifies the AQSS client.
 * @param[in]   p_ble_aqss_evt Pointer to the AQSS client event.
 */

/**@snippet [Handling events from the ble_aqss_c module] */
static void ble_aqss_c_evt_handler(
        ble_aqss_c_t *p_ble_aqss_c, ble_aqss_c_evt_t const *p_ble_aqss_evt) {
    ret_code_t err_code;

    switch (p_ble_aqss_evt->evt_type) {
    case BLE_AQSS_C_EVT_DISCOVERY_COMPLETE:
        NRF_LOG_INFO("Discovery complete.");
            err_code = ble_aqss_c_handles_assign(
                    p_ble_aqss_c, p_ble_aqss_evt->conn_handle, &p_ble_aqss_evt->handles);
        APP_ERROR_CHECK(err_code);

        err_code = ble_aqss_c_tx_notif_enable(p_ble_aqss_c);
        APP_ERROR_CHECK(err_code);
            NRF_LOG_INFO("Connected to AQ3xxSH device with AQSS service.");

            m_aq_device[p_ble_aqss_evt->conn_handle].conn_handle = p_ble_aqss_evt->conn_handle;
            m_aq_device[p_ble_aqss_evt->conn_handle].device_model = AQ_3xx_SH;


        break;

    case BLE_AQSS_C_EVT_AQSS_TX_EVT:
            ble_aqss_chars_received_data(
                    p_ble_aqss_c->conn_handle, p_ble_aqss_evt->p_data, p_ble_aqss_evt->data_len);
        break;

    case BLE_AQSS_C_EVT_DISCONNECTED:
        NRF_LOG_INFO("Disconnected AQSS.");
        scan_start();
        break;
    }
}


/**@brief Callback handling DIS client events.
 *
 * @details This function is called to notify the application of DIS client events.
 *
 * @param[in]   p_ble_aqts_c   DIS client handle. This identifies the DIS client.
 * @param[in]   p_ble_aqts_evt Pointer to the DIS client event.
 */
static void ble_disc_evt_handler(
        ble_dis_c_t *p_ble_dis_c, ble_dis_c_evt_t const *p_ble_dis_evt) {
    ret_code_t err_code;
    char arr_1[5];

    switch (p_ble_dis_evt->evt_type) {
        case BLE_DIS_C_EVT_DISCOVERY_COMPLETE:
            NRF_LOG_INFO("Discovery complete.");
            err_code = ble_dis_c_handles_assign(
                    p_ble_dis_c, p_ble_dis_evt->conn_handle, &p_ble_dis_evt->params.read_rsp);
            APP_ERROR_CHECK(err_code);

            err_code = ble_dis_c_read(p_ble_dis_c, BLE_DIS_C_SERIAL_NUM);
            APP_ERROR_CHECK(err_code);
            NRF_LOG_INFO("Connected to device with DIS Service.");
            break;
        case BLE_DIS_C_EVT_DIS_C_READ_RSP:
            NRF_LOG_INFO("Disc read");
            // todo:
            if (p_ble_dis_evt->params.read_rsp.char_type == BLE_DIS_C_SERIAL_NUM) {
                memcpy(m_aq_device[p_ble_dis_evt->conn_handle].device_id,
                       p_ble_dis_evt->params.read_rsp.content.string.p_data, 16);

                /* Start RSSI */
                sd_ble_gap_rssi_start(p_ble_dis_evt->conn_handle, 10, 5);

                if (m_aq_device[p_ble_dis_evt->conn_handle].device_model == AQ_1xx_WC) {
                    err_code = ble_aqts_c_tx_notif_enable(
                            get_toilet_le_dev_instance(p_ble_dis_evt->conn_handle));
                    APP_ERROR_CHECK(err_code);
                    NRF_LOG_INFO("Connected to AQ1xxWC device with AQTS Service.");
               }

                if (m_aq_device[p_ble_dis_evt->conn_handle].device_model == AQ_3xx_SH) {
                    err_code = ble_aqss_c_tx_notif_enable(
                            get_shower_le_dev_instance(p_ble_dis_evt->conn_handle));
                    APP_ERROR_CHECK(err_code);
                    NRF_LOG_INFO("Connected to AQ3xxSH device with AQSS service.");              

                }

            }
            if (p_ble_dis_evt->params.read_rsp.char_type == BLE_DIS_C_MODEL_NUM) {
                memcpy(m_aq_device[p_ble_dis_evt->conn_handle].device_model_name,
                       p_ble_dis_evt->params.read_rsp.content.string.p_data, 3);
            }          	

            break;
        case BLE_DIS_C_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected DIS.");
            break;
    }
}
/**@snippet [Handling events from the ble_nus_c module] */


/**@brief Function for handling the Aquai Shower Service Client errors.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void aqss_error_handler(uint32_t nrf_error) { APP_ERROR_HANDLER(nrf_error); }


/**@brief Function for handling the Aquai Toilet Service Client errors.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void aqts_error_handler(uint32_t nrf_error) { APP_ERROR_HANDLER(nrf_error); }

/**@brief Function for handling the DIS Client errors.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void disc_error_handler(uint32_t nrf_error) { APP_ERROR_HANDLER(nrf_error); }

/**@brief Function for initializing the Aquai Shower Service (AQSS) client. */
static void aqss_c_init(void) {
    ret_code_t       err_code;
    ble_aqss_c_init_t init;

    init.evt_handler   = ble_aqss_c_evt_handler;
    init.error_handler = aqss_error_handler;
    init.p_gatt_queue  = &m_ble_gatt_queue;

    uint32_t i = 0;
    for (i = 0; i < NRF_SDH_BLE_CENTRAL_LINK_COUNT; i++) {
        err_code = ble_aqss_c_init(&m_ble_aqss_c[i], &init);
        APP_ERROR_CHECK(err_code);
    }
}

/**@brief Function for initializing the Aquai Toilet Service (AQTS) client. */
static void aqts_c_init(void) {
    ret_code_t       err_code;
    ble_aqts_c_init_t init;

    init.evt_handler   = ble_aqts_c_evt_handler;
    init.error_handler = aqts_error_handler;
    init.p_gatt_queue  = &m_ble_gatt_queue;

    uint32_t i = 0;
    for (i = 0; i < NRF_SDH_BLE_CENTRAL_LINK_COUNT; i++) {
        err_code = ble_aqts_c_init(&m_ble_aqts_c[i], &init);
        APP_ERROR_CHECK(err_code);
    }
}

/**@brief Function for initializing the DIS client. */
static void dis_c_init(void) {
    ret_code_t err_code;
    ble_dis_c_init_t init;
    ble_dis_c_char_group_t char_group;
    ble_dis_c_char_type_t char_type[] = {BLE_DIS_C_MANUF_NAME, BLE_DIS_C_MODEL_NUM,
                                         BLE_DIS_C_SERIAL_NUM, BLE_DIS_C_HW_REV, BLE_DIS_C_FW_REV};

    char_group.size = 5;
    char_group.p_char_type = char_type;

    init.evt_handler = ble_disc_evt_handler;
    init.error_handler = disc_error_handler;
    init.p_gatt_queue = &m_ble_gatt_queue;
    init.char_group = char_group;
    

    uint32_t i = 0;
    for (i = 0; i < NRF_SDH_BLE_CENTRAL_LINK_COUNT; i++) {
        err_code = ble_dis_c_init(&m_ble_dis_c[i], &init);
        APP_ERROR_CHECK(err_code);
    }
}

void init_client_services(void) {
    /* initialize the Client Services */
    aqss_c_init();
    aqts_c_init();
    dis_c_init();

    /* Init test connection context */
    for (uint8_t i = 0; i < NRF_SDH_BLE_CENTRAL_LINK_COUNT; i++) {
        m_aq_device[i].p_ble_aqss_c = &m_ble_aqss_c[i];
        m_aq_device[i].p_ble_aqts_c = &m_ble_aqts_c[i];
        m_aq_device[i].p_ble_dis_c = &m_ble_dis_c[i];
        memset(m_aq_device[i].device_id, 0x00, 16);
        memset(m_aq_device[i].device_model_name, 0x00, 7);
        m_aq_device[i].conn_handle = 0xFFFF;
        // APP_ERROR_CHECK(err_code);
    }
}

#ifdef __cplusplus
}
#endif